/* Copyright (c) 1998, MpegTV LLC */
#ifndef _dith_im_h
#define _dith_im_h
#ifndef lint
static const char rcsid_dith_im_h[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/dith_im.h,v 1.4 1999/05/04 01:10:09 tristan Exp tristan $";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */


#include "mtvp/misc.h"
#include "mtvp/mode_api.h"
#include "mtvp/ts_api.h"
#include "mtvp/fifo_element.h"

/*--------------------------------------------------------------------------+
| Macros																	|
+--------------------------------------------------------------------------*/
/* temporal direction FORWARD/BACKWARD */
#define F			0	/* FORWARD */
#define B			1	/* BACKWARD */
#define FB			2	/* size of arrays indexed by F and B */

/* spatial direction HORIZONTAL/VERTICAL (used for prediction motion vect) */
#define HT			0	/* HORIZONTAL */
#define VT			1	/* VERTICAL (for MPEG-2 only) */
#define HV			2	/* size of arrays indexed by HT and VT */


/*--------------------------------------------------------------------------+
| Data types																|
+--------------------------------------------------------------------------*/
/* YUV is for MTVP_Y, MTVP_U and MTVP_V plane indexes */
#define MTVP_Y		0
#define MTVP_U		1
#define MTVP_V		2
typedef u_char		MTVP_YUV;

/* image plane */
typedef struct {
	u_int		width;
	u_int		height;
	u_char	*array;		/* ptr to plane data */
} MTVP_PLANE;

/* frame infos */
typedef struct {
	MTVP_VIDEO_CODING_TYPE	mode;				/* mode: MTVP_VIDEO_CODING_TYPE_INTRA, MTVP_VIDEO_CODING_TYPE_PREDICTIVE, MTVP_VIDEO_CODING_TYPE_INTERPOLATIVE */
	u_int	temporal_reference;	/* temporal reference */
	double	vbv_delay;			/* vbv_delay (in sec) */
	u_int 	f_code[FB][HV];		/* mc "f_code" [forward, backward][horiz, vert] */
	Bool 	full_pel[FB];		/* if True, motion vect [F, B] are full-pel */
	u_int	display_index;		/* index incremented in display order */
	MTVP_TS		pts;				/* systems presentation time stamp */
} MTVP_PMP_DATA_FRAME_INFOS;


/* image */
typedef struct {
	MTVP_FIFO_ELEMENT	fifo;
	int		refs;			/* reference counter */
	MTVP_PMP_DATA_FRAME_INFOS	frame_infos;
	Bool	decoded;		/* have slices been decoded ? */
	double	target_display_time; /* 0.0 means ASAP */
	double	estimated_display_time;
	void	(*free_func)(void *);
	MTVP_PLANE	plane[3];	/* Y, U, and V */
} MTVP_YUV_IMAGE;


/* dithered image in the format suitable to be displayed */
typedef struct {
	MTVP_YUV_IMAGE	*yuv_image;

	// src_x, src_y, src_width and src_height define the part of the source image that
	// will be converted by the internal 'dithering' and YUV->RGB
	// routines.  At this time, only the 16-bit RGB 1x1 zoom routine
	// uses those variables.  All other routines ignore them and always convert
	// the entire frame.
	//
	// If not set by VOH_dithered_image_lock, default will be
	// assumed.  The default corresponds to converting the entire
	// image.
	u_int   src_x;				/* in src pels */
	u_int   src_y;				/* in src pels */
	u_int   src_width;			/* in src pels */
	u_int   src_height;			/* in src pels */
	
	u_int   pitch;				/* in pels */
	u_int   bytes_per_pel;		/* number of bytes per pel */

	u_char *array;				/* ptr to dest pels */
	
	Bool	is_scaled_2x2;

} MTVP_VOH_DITHERED_IMAGE;

#endif /* _dith_im_h */
