/* Copyright (c) 1997, MpegTV LLC */
#ifndef _mtvp_player_hpp
#define _mtvp_player_hpp
#ifndef lint
static const char rcsid_mtvp_player_hpp[] = "$Header: /net/hopkins/home/tristan/mpeg/dec/RCS/g2a_api.h,v 1.1 1999/08/03 08:05:19 tristan Exp tristan $";
#endif

/*
 * Disclaimer of Warranty
 *
 * These software programs are available to the user without any license fee or
 * royalty on an "as is" basis.  Copyright-holder disclaims
 * any and all warranties, whether express, implied, or statuary, including any
 * implied warranties or merchantability or of fitness for a particular
 * purpose.  In no event shall the copyright-holder be liable for any
 * incidental, punitive, or consequential damages of any kind whatsoever
 * arising from the use of these programs.
 *
 * This disclaimer of warranty extends to the user of these programs and user's
 * customers, employees, agents, transferees, successors, and assigns.
 *
 */

#include "mtvp/mtvp.h"


/*--------------------------------------------------------------------------+
| External
+--------------------------------------------------------------------------*/
/* construction zone! this class is not fully defined yet */
class MTVP_Player {
private:
	int	synchronous_flag;
public:
	MTVP_PLAYER *player;
//	int	voh_handler_id;

	MTVP_Player() {
		player = (MTVP_PLAYER *)0;
		synchronous_flag = 0;
	}
	~MTVP_Player() {
		if (player) {
			mtvp_destroy_player(player);
		}
	}
	int MtvpRun(const char **mtvp_argv, const char *argv0, void *platform_ptr) {
		int status;
		status = mtvp_create_player(&player, mtvp_argv, argv0, platform_ptr);
		if (status) player = (MTVP_PLAYER *)0;
		return status;
	}
	void MtvpHandleNotificationMessage(MTVP_PMP_HEADER *msg, const MTVP_PMP_DATA_A2G *data) {

		switch (msg->type) {
			case MTVP_NOTIFY_RETURN_SUCCESS:
				MtvpOnNotifySuccess();
				break;
			case MTVP_NOTIFY_RETURN_FAILURE:
				MtvpOnNotifyFailure();
				break;
			case MTVP_NOTIFY_READY:
				MtvpOnNotifyReady();
				break;
			case MTVP_NOTIFY_QUIT:
				MtvpOnNotifyQuit();
				break;
			case MTVP_NOTIFY_STATE:
				MtvpOnNotifyState((MTVP_STATE)data->data_generic.data_int);
				break;
			case MTVP_NOTIFY_POS:
				MtvpOnNotifyPosition(data->data_generic.data_int);
				break;
			case MTVP_NOTIFY_LOOP_END:
				MtvpOnNotifyLoopEnd();
				break;
			case MTVP_NOTIFY_ACTUAL_FRAMERATE:
				MtvpOnNotifyActualFramerate(data->data_generic.data_double);
				break;
			case MTVP_NOTIFY_AUDIO_ENABLED:
				MtvpOnNotifyAudioEnabled();
				break;
			case MTVP_NOTIFY_AUDIO_DISABLED:
				MtvpOnNotifyAudioDisabled();
				break;
			case MTVP_NOTIFY_AUDIO_VOLUME:
				MtvpOnNotifyVolume(data->data_generic.data_double);
				break;
			case MTVP_NOTIFY_AUDIO_BALANCE:
				MtvpOnNotifyBalance(data->data_generic.data_double);
				break;
			case MTVP_NOTIFY_AUDIO_MUTE:
				MtvpOnNotifyMute();
				break;
			case MTVP_NOTIFY_VIDEO_DITHER_TYPE:
				MtvpOnNotifyDitherType((MTVP_DITHER_TYPE)data->data_generic.data_int);
				break;
			case MTVP_NOTIFY_VIDEO_DITHER_FLAGS:
				MtvpOnNotifyDitherFlags(data->data_generic.data_int);
				break;
			case MTVP_NOTIFY_INPUT_FILE_SIZE:
				MtvpOnNotifyFileSize((u_int)data->data_generic.data_int);
				break;
			case MTVP_NOTIFY_DISPLAY_READY:
				MtvpOnNotifyDisplayReady();
				break;
			case MTVP_NOTIFY_VIDEO_INFOS:
				MtvpOnNotifyVideoInfos((const MTVP_PMP_DATA_VIDEO_INFOS *)data);
				break;
			case MTVP_NOTIFY_GOP_INFOS:
				MtvpOnNotifyGOPInfos((const MTVP_PMP_DATA_GOP_INFOS *)data);
				break;
			case MTVP_NOTIFY_AUDIO_INFOS:
				MtvpOnNotifyAudioInfos((const MTVP_PMP_DATA_AUDIO_INFOS *)data);
				break;
			case MTVP_NOTIFY_FRAME_INFOS:
				MtvpOnNotifyFrameInfos((const MTVP_PMP_DATA_FRAME_INFOS *)data);
				break;
			case MTVP_NOTIFY_PACK_INFOS:
				MtvpOnNotifyPackInfos((const MTVP_PMP_DATA_PACK_INFOS *)data);
				break;
			case MTVP_NOTIFY_VOH_FEEDBACK:
				MtvpOnNotifyVOHFeedback((const MTVP_VOH_FEEDBACK *)data, msg->size);
				break;
			case MTVP_NOTIFY_SIH_FEEDBACK:
				MtvpOnNotifySIHFeedback((const MTVP_SIH_FEEDBACK *)data, msg->size);
				break;
			case MTVP_NOTIFY_MPEG2_VIDEO:
				MtvpOnNotifyMpeg2Video();
				break;
			case MTVP_NOTIFY_BAD_STREAM:
				MtvpOnNotifyBadStream();
				break;
			case MTVP_NOTIFY_CANNOT_PLAY_AUDIO:
				MtvpOnNotifyCannotPlayAudio(data->data_generic.data_int);
				break;
			case MTVP_NOTIFY_CANNOT_SHOW_VIDEO:
				MtvpOnNotifyCannotShowVideo(data->data_generic.data_int);
				break;
			case MTVP_NOTIFY_DISPLAY_THREAD_ID:
#ifdef WIN32
				MtvpOnNotifyDisplayThreadId((HANDLE)data->data_generic.data_int);
#else
				MtvpOnNotifyDisplayThreadId((pthread_t *)data);
#endif
				break;
		}
	}
	void MtvpSetSynchronousMode() {
		synchronous_flag = MTVP_PMP_FLAG_SYNCHRONOUS;
	}
	int MtvpSendCommandCustom(MTVP_COMMAND command, const void *ptr, u_int size) {
		return mtvp_send_command_custom(player, command, synchronous_flag, ptr, size);
	}
private:
	int SendCommandVoid(MTVP_COMMAND command) {
		return mtvp_send_command_void(player, command, synchronous_flag);
	}
	int SendCommandInt(MTVP_COMMAND command, int data) {
		return mtvp_send_command_int(player, command, synchronous_flag, data);
	}
	int SendCommandString(MTVP_COMMAND command, const char *string) {
		return mtvp_send_command_string(player, command, synchronous_flag, string);
	}
	int SendCommandDouble(MTVP_COMMAND command, double d) {
		return mtvp_send_command_double(player, command, synchronous_flag, d);
	}
public:
	int MtvpGetCommandStatus(int *status, void **value) {
		int ret = mtvp_get_command_status(player, status, value);
		synchronous_flag = 0;
		return ret;
	}
	int MtvpOpenStreamURL(const char *url) {
		return SendCommandString(MTVP_COMMAND_OPEN_STREAM_URL, url);
	}
	int MtvpCloseStream() {
		return SendCommandVoid(MTVP_COMMAND_CLOSE_STREAM);
	}
	int MtvpSetState(MTVP_STATE state) {
		return SendCommandInt(MTVP_COMMAND_STATE, state);
	}
	int MtvpPlay() {
		return MtvpSetState(MTVP_STATE_PLAYING);
	}
	int MtvpPause() {
		return MtvpSetState(MTVP_STATE_PAUSED);
	}
	int MtvpSteps(u_int n) {
		return SendCommandInt(MTVP_COMMAND_STEPS, n);
	}
	int MtvpSeek(u_int pos) {
		return SendCommandInt(MTVP_COMMAND_SEEK, (u_int)pos);
	}
	int MtvpSeekToEnd() {
		return SendCommandVoid(MTVP_COMMAND_END);
	}
	int MtvpSetAudioChannel(u_int channel) {
		return SendCommandInt(MTVP_COMMAND_AUDIO_CHANNEL, channel);
	}
	int MtvpSetAudioQuality(u_int quality) {
		return SendCommandInt(MTVP_COMMAND_AUDIO_QUALITY, quality);
	}
	int MtvpLoop(u_int yes) {
		return SendCommandInt(MTVP_COMMAND_LOOP, yes);
	}
	int MtvpAutoPlay(u_int yes) {
		return SendCommandInt(MTVP_COMMAND_AUTO_PLAY, yes);
	}
	int MtvpMute(u_int yes) {
		return SendCommandInt(MTVP_COMMAND_AUDIO_MUTE, yes);
	}
	int MtvpRepaint() {
		return SendCommandVoid(MTVP_COMMAND_VIDEO_REPAINT);
	}
	int MtvpFocus() {
		return SendCommandVoid(MTVP_COMMAND_VIDEO_FOCUS);
	}
	int MtvpQuit() {
		return SendCommandVoid(MTVP_COMMAND_QUIT);
	}
	int MtvpSetDisplayWindow(int window_id) {
		return SendCommandInt(MTVP_COMMAND_WINDOW_ID, window_id);
	}
	int MtvpSetGammaCorrection(double x) {
		return SendCommandDouble(MTVP_COMMAND_VIDEO_GAMMA, x);
	}
	int MtvpSetChromaSaturation(double x) {
		return SendCommandDouble(MTVP_COMMAND_VIDEO_CHROMA_SATURATION, x);
	}
	int MtvpSetSyncType(MTVP_SYNC_TYPE sync_type) {
		return SendCommandInt(MTVP_COMMAND_SYNC_TYPE, (int)sync_type);
	}
//	int SetFramerate(double rate) {
//		return SendCommandDouble(MTVP_COMMAND_SYNC_FRAMERATE, rate);
//	}
	int MtvpSetRateFactor(double rate) {
		return SendCommandDouble(MTVP_COMMAND_SYNC_RATE_FACTOR, rate);
	}
	int MtvpSetVolume(double volume) {
		return SendCommandDouble(MTVP_COMMAND_AUDIO_VOLUME, volume);
	}
	int MtvpSetBalance(double balance) {
		return SendCommandDouble(MTVP_COMMAND_AUDIO_BALANCE, balance);
	}
	int MtvpSetAudioControls(MTVP_AUDIO_CONTROLS n) {
		return SendCommandInt(MTVP_COMMAND_AUDIO_CONTROLS, (int)n);
	}
	int MtvpLinkSIH(const char *path) {
		return SendCommandString(MTVP_COMMAND_SIH_LINK, path);
	}
	int MtvpInitSIH(int handler_id) {
		return SendCommandInt(MTVP_COMMAND_SIH_INIT, handler_id);
	}
	int MtvpFreeSIH(int handler_id) {
		return SendCommandInt(MTVP_COMMAND_SIH_FREE, handler_id);
	}
	int MtvpControlSIH(const MTVP_SIH_CNTL *data, u_int size) {
		return MtvpSendCommandCustom(MTVP_COMMAND_SIH_CNTL, data, size);
	}
	int MtvpLinkVOH(const char *path) {
		return SendCommandString(MTVP_COMMAND_VOH_LINK, path);
	}
	int MtvpInitVOH(int handler_id) {
		return SendCommandInt(MTVP_COMMAND_VOH_INIT, handler_id);
	}
	int MtvpFreeVOH(int handler_id) {
		return SendCommandInt(MTVP_COMMAND_VOH_FREE, handler_id);
	}
	int MtvpSelectVOH(int handler_id) {
		return SendCommandInt(MTVP_COMMAND_VOH_SELECT, handler_id);
	}
	int MtvpControlVOH(const MTVP_VOH_CNTL *data, u_int size) {
		return MtvpSendCommandCustom(MTVP_COMMAND_VOH_CNTL, data, size);
	}
	int MtvpGetPosition() {
		return SendCommandVoid(MTVP_COMMAND_GET_POS);
	}
	int MtvpGetActualFramerate() {
		return SendCommandVoid(MTVP_COMMAND_GET_ACTUAL_FRAMERATE);
	}
	int MtvpGetNumberOfFramesPlayed() {
		return SendCommandVoid(MTVP_COMMAND_GET_NUMBER_OF_FRAMES_PLAYED);
	}
	int MtvpMsgSelect(int msg_mask) {
		return SendCommandInt(MTVP_COMMAND_MSG_SELECT, msg_mask);
	}
	int MtvpSuspendVideo() {
		return SendCommandVoid(MTVP_COMMAND_VIDEO_SUSPEND);
	}
	int MtvpResumeVideo() {
		return SendCommandVoid(MTVP_COMMAND_VIDEO_RESUME);
	}
	int MtvpDitherSpec(int dither_spec) {
		return SendCommandInt(MTVP_COMMAND_DITHER_SPEC, dither_spec);
	}
protected:
	virtual void MtvpOnNotifySuccess() {}
	virtual void MtvpOnNotifyFailure() {}
	virtual void MtvpOnNotifyReady() {}
	virtual void MtvpOnNotifyQuit() {}
	virtual void MtvpOnNotifyState(MTVP_STATE state) {}
	virtual void MtvpOnNotifyPosition(int pos) {}
	virtual void MtvpOnNotifyLoopEnd() {}
	virtual void MtvpOnNotifyActualFramerate(double x) {}
	virtual void MtvpOnNotifyAudioEnabled() {}
	virtual void MtvpOnNotifyAudioDisabled() {}
	virtual void MtvpOnNotifyVolume(double x) {}
	virtual void MtvpOnNotifyBalance(double x) {}
	virtual void MtvpOnNotifyMute() {}
	virtual void MtvpOnNotifyDitherType(MTVP_DITHER_TYPE type) {}
	virtual void MtvpOnNotifyDitherFlags(int flags) {}
	virtual void MtvpOnNotifyFileSize(u_int size) {}
	virtual void MtvpOnNotifyDisplayReady() {}
	virtual void MtvpOnNotifyVideoInfos(const MTVP_PMP_DATA_VIDEO_INFOS *ptr) {}
	virtual void MtvpOnNotifyGOPInfos(const MTVP_PMP_DATA_GOP_INFOS *ptr) {}
	virtual void MtvpOnNotifyAudioInfos(const MTVP_PMP_DATA_AUDIO_INFOS *ptr) {}
	virtual void MtvpOnNotifyFrameInfos(const MTVP_PMP_DATA_FRAME_INFOS *ptr) {}
	virtual void MtvpOnNotifyPackInfos(const MTVP_PMP_DATA_PACK_INFOS *ptr) {}
	virtual void MtvpOnNotifyVOHFeedback(const MTVP_VOH_FEEDBACK *data, u_int size) {}
	virtual void MtvpOnNotifySIHFeedback(const MTVP_SIH_FEEDBACK *data, u_int size) {}
	virtual void MtvpOnNotifyMpeg2Video() {}
	virtual void MtvpOnNotifyBadStream() {}
	virtual void MtvpOnNotifyCannotPlayAudio(int status) {}
	virtual void MtvpOnNotifyCannotShowVideo(int status) {}
#ifdef WIN32
	virtual void MtvpOnNotifyDisplayThreadId(HANDLE thread_id) {}
#else
	virtual void MtvpOnNotifyDisplayThreadId(pthread_t *thread_id) {}
#endif
};



#endif /* _mtvp_player_hpp */
