
#if !defined(_pockettv_h)
#define _pockettv_h

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include "resource.h"

#include "mtvp/mtvp.h"

#include "mtvp_player_win32.hpp"

/*--------------------------------------------------------------------------+
| Configuration flags
+--------------------------------------------------------------------------*/
// #define NO_REGISTRATION	// remove registration code
// #define NO_GLOBAL		// remove the PocketTV *Player global (that's the only one)
// #define USE_FULLSCREEN	// add code that deals with full screen (only works on ipaq)

/*--------------------------------------------------------------------------+
| PocketTV Class
+--------------------------------------------------------------------------*/
#define POCKETTV_VERSION_MAJOR		0
#define POCKETTV_VERSION_MINOR		3
#define POCKETTV_VERSION_REVISION	6

typedef enum {
	UNDIM_TYPE_GUESS = -1,
	UNDIM_TYPE_NONE = 0,
	UNDIM_TYPE_HP_540 = 1,
	UNDIM_TYPE_CASIO_115 = 2
} UNDIM_TYPE;

typedef struct {
	UNDIM_TYPE	type;
	unsigned int		timer_interval;
} UNDIM;

class PocketTV : public MTVP_Player_win32
{
private:
	HWND				m_MenuCommandBar;		// The menu command bar handle
#ifndef POCKET_PC
	HWND				m_CommandBands;		// The command band handle
#endif
	HWND				m_ControlsCommandBar;	// The command bar handle that hold the slider
	HWND				m_StatusWindow;		// The status window handle
	bool				m_ShowStatusBar;	// Show/hide status bar
	HMENU				m_Menu;				// The menu bar handle
	TCHAR			   *m_StreamName;		// URL of opened stream
	TCHAR			   *m_NavBarTitle;		// name of opened stream as shown in navigation bar (when playing)
	bool				m_DisplayReady;		// turns True once first image is displayed after a seek
	int					m_PendingSeek;		// -1 or pending seek position
	HWND				m_SeekSlider;		// The track slider handle
	HWND				m_VolumeSlider;		// The volume slider handle
	MTVP_STATE			m_StateBeforeSeeking;
	MTVP_STATE			m_State;			// current state on player
	int					m_WasPlayingBeforeBecommingInactive; // -1 means unknown (i.e. not yet set)
//	MTVP_STATE			m_StateBeforeCancelMode; // -1 means unknown (i.e. not yet set)
	bool				m_CancelModePending;
	bool				m_SettingVolume;	// used by volume slider procedure
	bool				m_FastForward;		// True if playing fast forward
    WNDPROC				m_VolumeSliderParentOriginalProc;
	WNDPROC				m_SeekSliderParentOriginalProc;
	RECT				m_TargetRect;			// Rectangle for video
	double				m_FilesizeOverTrackSliderMax;
	double				m_TrackSliderMaxOverFilesize; // ratio of TrackSliderMax divided by Filesize
	bool				m_Seeking;			// True means we are currently seeking
	LONGLONG			m_LastPositionTimerTime;	// used by the position timer
	bool				m_PlayerReady;		// turns True once the player is fully initialized
	bool				m_Muted;			// audio is muted
	int					m_VOHHandlerId;
	bool				m_IgnoreCancelMode;
//	bool				m_Done;				// set when user clicks the OK button in the Nav Bar
	bool				m_AutoPlay;
	bool				m_FastVideo;		// Will use fast video if possible
//	bool				m_FastVideoDisabled;	// Fast video is not available
	VOH_WIN32_STRETCH	m_Stretch;
	UNDIM_TYPE			m_UndimType;
	unsigned int		m_VolumeRange;		// divisor factor for the volume range (volume is too loud on the iPaq)
	double				m_Volume;			// current Volume (<0 means unknown)
	HANDLE				m_DisplayThread;			
	bool				m_StreamOpened;
	bool				m_Registered;
	bool				m_InOpenDialog;
#ifndef POCKET_PC
	RECT				m_StreamNameRect;	// Rectangle for file name
#endif
	bool				m_GapiExchangeXY;
	bool				m_DisableResumeVideo;
#ifdef USE_FULLSCREEN
	HWND				m_FullscreenWindow;
	bool				m_PlayFullscreen;
#endif
	double				m_RateFactor;				// playback rate factor for >>

	MTVP_PMP_DATA_PACK_INFOS m_PackInfos;
	MTVP_PMP_DATA_VIDEO_INFOS m_VideoInfos;
	unsigned int		m_Filesize;
	unsigned int		m_Position;
	double				m_ActualFramerate;

public:
	HWND 				m_Window;
	HINSTANCE			m_Instance;			// The current instance
	HWND				m_VolumeDialog;		// The volume slider handle
	const TCHAR		   *m_URLName;			// name of last opened URL
	DWORD				m_Code;				// registration code
#ifdef USE_FULLSCREEN
	bool				m_Fullscreen;
#endif
	bool				m_Dither;
	int					m_DitherType;	// MTVP_DITHER_SPEC_12BIT or MTVP_DITHER_SPEC_15BIT

public:
    PocketTV(const char **mtvp_argv, HINSTANCE instance, HWND hWnd) :
		MTVP_Player_win32(mtvp_argv, instance),
		m_Instance(instance),
		m_Window(hWnd),
		m_PendingSeek(-1),
		m_WasPlayingBeforeBecommingInactive(-1),
//		m_StateBeforeCancelMode(MTVP_STATE_UNKNOWN),
		m_CancelModePending(false),

		m_MenuCommandBar(0),
#ifndef POCKET_PC
		m_CommandBands(0),
#endif
		m_ControlsCommandBar(0),
		m_StatusWindow(0),
		m_ShowStatusBar(false),
		m_Menu(0),
//		m_StreamNameRect(0),
		m_StreamName(NULL),
		m_NavBarTitle(NULL),
		m_DisplayReady(false),
		m_SeekSlider(0),
		m_VolumeSlider(0),
		m_StateBeforeSeeking(MTVP_STATE_PAUSED),
		m_State(MTVP_STATE_PAUSED),
		m_SettingVolume(false),
		m_FastForward(false),
		m_VolumeSliderParentOriginalProc(NULL),
		m_SeekSliderParentOriginalProc(NULL),
//		m_TargetRect(0),
		m_FilesizeOverTrackSliderMax(0.0),
		m_TrackSliderMaxOverFilesize(0.0),
		m_Seeking(false),
		m_LastPositionTimerTime(0),
		m_PlayerReady(false),
		m_Muted(false),
		m_VOHHandlerId(MTVP_VOH_HANDLER_ID_WIN32),
		m_IgnoreCancelMode(false),
//		m_Done(false),
		m_FastVideo(false),
//		m_FastVideoDisabled(false),
		m_AutoPlay(false),
		m_Stretch(VOH_WIN32_STRETCH_AUTO),
		m_UndimType(UNDIM_TYPE_GUESS),
		m_VolumeRange(1),
		m_Volume(-1.0),
		m_DisplayThread(0),
		m_StreamOpened(false),
		m_Registered(false),
		m_InOpenDialog(false),
		m_DisableResumeVideo(false),
#ifdef USE_FULLSCREEN
		m_FullscreenWindow(NULL),
		m_PlayFullscreen(false),
		m_Fullscreen(false),
#endif
		m_DitherType(MTVP_DITHER_SPEC_NONE),
		m_Dither(false),
		m_RateFactor(8),


//		m_PackInfos(0),
//		m_VideoInfos(0),
		m_Filesize(0),
		m_Position(0),
		m_ActualFramerate(0.0),
		m_VolumeDialog(0),
		m_URLName(NULL),
		m_Code(0),
		m_GapiExchangeXY(false)
	{
#ifndef POCKET_PC
		memset((void *)&m_StreamNameRect, 0, sizeof(m_StreamNameRect));
#endif
		memset((void *)&m_TargetRect, 0, sizeof(m_TargetRect));
		memset((void *)&m_PackInfos, 0, sizeof(m_PackInfos));
		memset((void *)&m_VideoInfos, 0, sizeof(m_VideoInfos));
	};
   ~PocketTV();

   LRESULT HandleWindowsMessage(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
   LRESULT SeekSliderProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
   LRESULT VolumeSliderProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
   BOOL VolumeDialogProc(HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam);
	void StartPlayTimers();
	void StopPlayTimers();
	void InitControls();
	void DestroyControls();
	void SetStreamControlsState(bool enabled);
	void ActivateStreamControls();
	void DeactivateStreamControls();
	int OpenURL(const TCHAR *url);
	int OpenFile();
	void CloseStream();
	void SetVideoStretch0(VOH_WIN32_STRETCH stretch);
	void SetVideoStretch(VOH_WIN32_STRETCH stretch);
	void SetAudioChannel(MTVP_AUDIO_CHANNEL channel);
	void SetAudioQuality(MTVP_AUDIO_QUALITY quality);
	void Loop(bool yes);
	void AutoPlay(bool yes);
	void ShowStatusBar(bool yes);
	int GapiInit();
	void GapiSetVideoTargetRect();
	bool Gapi0(bool yes, bool sync);
	void Gapi(bool yes);
//	int GapiSuspend();
//	int GapiResume();
	void SetState(MTVP_STATE state);
	void ReflectState();
	bool WasPlaying();
	int GetElapsedSeconds(unsigned int pos);
	void GetPositionParameters();
	void ShowTimeTotal();
	void ShowTimeElapsed();
	void ShowTimePosition(unsigned int pos);
	void ShowActualFramerate();
	void ShowBitrate();
	void MySeek(unsigned int pos);
	void KeepAlive();
	void CalculateVideoTargetRect();
	void SetVideoTargetRect();
	void UpdateVideoTargetRect();
	void UpdateSlidersSize();
	void UpdateSliderSize(HWND slider, HWND command_bar);
	HWND CreateSlider(HWND window, DWORD style, int x, int y, int width, int height, WNDPROC *window_original_proc, WNDPROC slider_proc);
	HWND CreateSliderInCommandBar(HWND command_bar, unsigned int last_button_index, WNDPROC *command_bar_original_proc, WNDPROC slider_proc);
	void MySetVolume(unsigned int volume);
	void ErrorBoxEx(HWND window, int ids_window_caption_error, const TCHAR *format, const va_list *args);
	void ErrorBox(HWND window, int ids_window_caption_error, const TCHAR *format, ...);
	void SetMute(bool yes);
#ifdef POCKET_PC
	void KeepBright();
	HWND CreateMenuBar(HWND hWnd, int nToolBarId, int dwFlags);
#endif
#ifndef NO_REGISTRATION
	LONG InitializeRegistrationCode();
	LONG GetRegistrationKey(DWORD *pKey);
	LONG SetRegistrationKey(DWORD reg_key);
	void InitRegistration();
#endif
	void SetUndimType(UNDIM_TYPE type);
	void SetVolumeRange(unsigned int range);
	void DialogAboutEasterEgg(HWND hWnd);
#ifdef USE_FULLSCREEN
	void Fullscreen(bool yes);
	void FullscreenKillFocus();
	void FullscreenSetFocus();
	void PlayFullscreen(bool yes);
#endif
//	int SuspendVideo0(unsigned int n);
//	int ResumeVideo0(unsigned int n);
	int SuspendVideo();
	int ResumeVideo();
	int Dither(bool yes);
	void SetDither(bool yes);
	void DitherType(int dither);
	int EnableDither(VOH_WIN32_STRETCH stretch);
	void RateFactor(int rate_factor);
	void SetRateFactor(int rate_factor);

	void MtvpOnNotifyState(MTVP_STATE state);
	void MtvpOnNotifyPackInfos(const MTVP_PMP_DATA_PACK_INFOS *ptr);
	void MtvpOnNotifyVideoInfos(const MTVP_PMP_DATA_VIDEO_INFOS *ptr);
	void MtvpOnNotifyFileSize(unsigned int size);
	void MtvpOnNotifyActualFramerate(double x);
	void MtvpOnNotifyVolume(double x);
	void MtvpOnNotifyPosition(int pos);
	void MtvpOnNotifyDisplayReady();
	void MtvpOnNotifyCannotShowVideo(int status);
	void MtvpOnNotifyCannotPlayAudio(int status);
	void MtvpOnNotifyReady();
	void MtvpOnNotifyDisplayThreadId(HANDLE thread);
};

/*--------------------------------------------------------------------------+
| Globals
+--------------------------------------------------------------------------*/
#ifdef NO_GLOBALS
// no globals!
#else
extern PocketTV *Player;
#endif

/*----------------------------------------------------------------------
|       prototypes
+---------------------------------------------------------------------*/

/*----------------------------------------------------------------------
|       macros
+---------------------------------------------------------------------*/
#ifdef NO_GLOBALS

#define POCKETTV_HANDLE_WINDOW_MESSAGE(object_class,                     \
                                 handler,                          \
                                 window,                           \
						         message,                          \
						         wparam,                           \
						         lparam)                           \
{                                                                  \
	object_class *target;                                          \
    target = (object_class *)GetWindowLong(window, GWL_USERDATA);  \
    if (target) {												   \
        return target->##handler##(window,                         \
		                           message,                        \
		 	    				   wparam,                         \
								   lparam);                        \
	} else {                                                       \
		return DefWindowProc(window, message, wparam, lparam);     \
	}                                                              \
}

#else /* not NO_GLOBALS */

#define POCKETTV_HANDLE_WINDOW_MESSAGE(object_class,                     \
                                 handler,                          \
                                 window,                           \
						         message,                          \
						         wparam,                           \
						         lparam)                           \
{                                                                  \
	PocketTV *target = Player;                                     \
    if (target) {												   \
        return target->##handler##(window,                         \
		                           message,                        \
		 	    				   wparam,                         \
								   lparam);                        \
	} else {                                                       \
		return DefWindowProc(window, message, wparam, lparam);     \
	}                                                              \
}

#endif /* not NO_GLOBALS */

extern "C" {
extern LRESULT CALLBACK DialogAbout(HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam);
}

#endif // !defined(_pockettv_h)
